import java.net.*;
import java.io.*;


public class FTPclient {

private BufferedReader bufferRead;
private BufferedWriter bufferWrite;
private Socket socketControl;
private String strReplyCode;
private Writer bwLog = null;
public static final boolean typeI = true;
public static final boolean typeA = false;
private boolean boFileType = typeA;

	public FTPclient(String strDstHost,int intContrPort,Writer bw) throws IOException
	{
		socketControl = new Socket(strDstHost,intContrPort);
		InputStream isRead= socketControl.getInputStream();
		bufferRead = new BufferedReader(new InputStreamReader(isRead));
		OutputStream osWrite= socketControl.getOutputStream();
		bufferWrite = new BufferedWriter(new OutputStreamWriter(osWrite));
		bwLog = bw;
		getReply();
	}
	//SEND COMMAND TO FTP SERVER
	public void sendCommand(String strCommand) throws IOException
	{
		bufferWrite.write(strCommand + "\r\n");
		bufferWrite.flush();
		if(bwLog != null)
			bwLog.write(strCommand + "\r\n");
	}
	//LOGIN
	public void login(String strUSER,String strPASS) throws IOException
	{
		sendCommand("USER " + strUSER);
		getReply();	
		sendCommand("PASS " + strPASS);
		getReply();
	}
	
	//LIST
	public String[] list() throws IOException
	{
		boolean boolPrev = boFileType;
		setType(typeA);
		FTPDataSocket socketData = new FTPDataSocket(this);
		sendCommand("LIST");
		getReply();
		String [] strResult =  socketData.read();
		setType(boolPrev);
		return strResult;
	}
	//PRINT WORKING DIRECTORY
	public String pwd() throws IOException
	{
		sendCommand("PWD");
		String strPWD = getReply();
		String[] strArray = strPWD.split("\"");
		return strArray[1];	
	}
	//MAKE DIRECTORY
	public void mkd(String strMKDIR) throws IOException
	{
		sendCommand("MKD " + strMKDIR);
		getReply();
	}
	//REMOVE DIRECTORY
	public void rmd(String strRMDIR) throws IOException
	{
		sendCommand("RMD " + strRMDIR);
		getReply();
	}
	//CHANGE WORKING DIRECTORY
	public void cwd(String strChangePath) throws IOException
	{
		sendCommand("CWD " + strChangePath);
		getReply();
	}
	//RENAME
	public void rename(String strRNFR,String strRNTO) throws IOException
	{
		sendCommand("RNFR " + strRNFR);
		getReply();
		sendCommand("RNTO " + strRNTO);
		getReply();
	}
	//CHANGE TO PARENT DIRECTORY
	public void cdup() throws IOException
	{
		sendCommand("CDUP");
		getReply();
	}
	//DELETE THE FILE SPECIFIED IN THE PATHNAME
	public void dele(String strRemoteFile) throws IOException
	{
		sendCommand("DELE " + strRemoteFile);
		getReply();
	}
	//DataType:ASCII or IMAGE
	public void setType(boolean boolDataType) throws IOException
	{
		boFileType = boolDataType;
		if ( boFileType == typeI )
			sendCommand("TYPE I");
		else
			sendCommand("TYPE A");
		getReply();
	}
	//GET DataType:ASCII or IMAGE
	public boolean getFileType()
	{
		return boFileType;
	}
	//RETRIEVE strRemote => the remote File to be download
	public byte[] retr(String strRemote) throws IOException
	{
		FTPDataSocket socketRETR = new FTPDataSocket(this);
		sendCommand("RETR " + strRemote);
		getReply();
		if(boFileType == typeA)
			return socketRETR.readASCII();
		else
			return socketRETR.readIMAGE();
	}
	//STORE strRemote => the remote File Name to be save
	public void stor(String strRemote,BufferedReader br) throws IOException
	{
		FTPDataSocket socketSTOR = new FTPDataSocket(this);
		sendCommand("STOR " + strRemote);
		getReply();
		socketSTOR.write(br);
	}
	//FTP REPLIES
	public String getReply() throws IOException
	{
		String strReply = bufferRead.readLine();	
		strReplyCode = strReply.substring(0,3); //eTӼƦr
		if(strReply.charAt(3)=='-') //pGOh檺reply Ʀr᭱|-
		{
			
			String strLine = bufferRead.readLine();
			while(!strLine.substring(0,3).equals(strReplyCode))
			{
				strReply = strReply + "\r\n" +strLine;
				strLine = bufferRead.readLine();	
			}
			strReply = strReply + "\r\n"+ strLine;
		}
		if(bwLog != null)
		{
			bwLog.write(strReply + "\r\n");
			bwLog.flush();
		}
		return strReply;
	}
	
	public String getReplyCode() throws IOException
	{
		return strReplyCode;	
	}
	//LOGOUT
	public void quit() throws IOException
	{
		sendCommand("QUIT");
		socketControl.close();
		
	}

}
